#!/bin/bash

#
# Name: vnx_download_rootfs
#
# Description: download, uncompress and optionally create links to root filesystems 
#               for VNX to current directory
#
# This file is a module part of VNX package.
#
# Authors: Jorge Somavilla (somavilla@dit.upm.es), David Fernández (david@dit.upm.es)
# Copyright (C) 2012,   DIT-UPM
#           Departamento de Ingenieria de Sistemas Telematicos
#           Universidad Politecnica de Madrid
#           SPAIN
#           
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# An online copy of the licence can be found at http://www.gnu.org/copyleft/gpl.html
#

#
# Constants
#
VNXACED_VER='2.0b.2053'
VNXACED_BUILT='23/05/2012_22:20'
#fsdir="/usr/share/vnx/filesystems"
vnx_rootfs_repo="http://vnx.dit.upm.es/vnx/filesystems/"
create_sym_link=""

USAGE="
vnx_download_rootfs: download, uncompress and optionally create links to 
                     root filesystems for VNX to current directory

Usage: vnx_download_rootfs [-l] [-r <rootfsname>]

Options:    -l -> creates a "rootfs_*" link to the root filesystem download
            -r -> use it to batch download a specific root filesystem
            when no options invoked vnx_download_rootfs starts in interactive mode            
"

# Variables
rootfs_links_array=''
HEADER="
----------------------------------------------------------------------------------
Virtual Networks over LinuX (VNX) -- http://www.dit.upm.es/vnx - vnx@dit.upm.es
Version: $VNXACED_VER ($VNXACED_BUILT)
----------------------------------------------------------------------------------
"



# Return name of symbolic link for a root_fs
function makeLinkName {
   if [[ $1 == *ubuntu* ]] ; then
      if [[ $1 == *gui* ]] ; then
         echo "rootfs_ubuntu-gui"
      else
         echo "rootfs_ubuntu"
      fi
   fi
   if [[ $1 == *freebsd* ]] ; then
      if [ $1 == *gui* ] ; then
         echo "rootfs_freebsd-gui"
      else
         echo "rootfs_freebsd"
      fi
   fi
   if [[ $1 == *fedora* ]] ; then
      if [[ $1 == *gui* ]] ; then
         echo "rootfs_fedora-gui"
      else
         echo "rootfs_fedora"
      fi
   fi
   if [[ $1 == *centos* ]] ; then
      if [[ $1 == *gui* ]] ; then
         echo "rootfs_centos-gui"
      else
         echo "rootfs_centos"
      fi
   fi
   if [[ $1 == *uml* ]] ; then
      if [[ $1 == *debian* ]] ; then
         echo "rootfs_tutorial"
      else
         echo "rootfs_light"
      fi
   fi
}


function install {

    if [ ! $rootfs_bzname = "" ] ; then
    
        # Rootfs specified: try to download and install it
        if [ -e $rootfs_bzname ]; then
            echo "Compressed root filesystem $rootfs_bzname already exists." 
            echo -n "Do you want to overwrite it (y/n)? "
            read choice
            if [ "$choice" == "y" -o  "$choice" = "Y" ] ; then
                echo "Deleting ${rootfs_bzname}..."
                rm -v ./$rootfs_bzname 
            else
                echo "Exiting..."
                exit 0
            fi
        fi

        myurl=(${vnx_rootfs_repo}${rootfs_bzname})
        echo "myurl=$myurl"
        error404=""
        #cd $fsdir
        wget $myurl || error404="true"
        #echo $error404
        if [ "$error404" == "true" ] ; then
            echo "------------------------------------------------------------------------"
            echo "ERROR: Root filesystem ($rootfs_name) not found in repository"
            echo "       Use 'vnx_download_rootfs -s' to see the list of rootfs available"
            echo "------------------------------------------------------------------------"
            #install_interactive
            exit
        fi
        
        if [ -e $rootfs_name ]; then
            echo "Uncompressed root filesystem $rootfs_name already exists."
            echo -n "Do you want to overwrite it (y/n)? "
            read choice
            if [ "$choice" == "y" -o  "$choice" = "Y" ] ; then
                echo "Deleting ${rootfs_name}..."
                rm -v ./$rootfs_name 
            else
                echo "Exiting..."
                exit 0
            fi
        fi
        echo ""
        echo "Extracting $rootfs_bzname..."
      
        bunzip2 $rootfs_bzname
        if [ "$create_sym_link" == "yes" ] ; then
            # Create symbolic link if -l
            chosen=$(echo $rootfs_bzname | sed -e "s/.bz2//g")
            linkname=$(makeLinkName $chosen)
            echo "Creating simbolic link: $linkname"
            ln -s $chosen $linkname
        fi
        echo "------------------------------------------------------------------------"
        echo "$rootfs_name successfully installed."
        echo "------------------------------------------------------------------------"
    else
        install_interactive
    fi
}


function show_rootfs_array {

    OLD_IFS=$IFS; IFS=$'\n'
    rootfs_links_array=($(curl $vnx_rootfs_repo -s | w3m -dump -T text/html | grep bz2))
    IFS=$OLD_IFS

    echo "Num   Rootfs name                                    Date        Size"
    echo "-----------------------------------------------------------------------"

    # Show filesystems on server in columns
    for (( i = 0 ; i < ${#rootfs_links_array[@]} ; i++ ))
    do
        OLD_IFS=$IFS; IFS=$'\n'
        NUM=$(printf "%-3s" "$i") 
        echo ${rootfs_links_array[$i]} | sed -e "s/\[ \]/$NUM/g" | column
        IFS=$OLD_IFS
    done
    echo "-----------------------------------------------------------------------"

}

function install_interactive {

    # show filesystems in server and ask user to choose
    OLD_IFS=$IFS; IFS=$'\n'
    rootfs_links_array=($(curl $vnx_rootfs_repo -s | w3m -dump -T text/html | grep bz2))
    IFS=$OLD_IFS

    while true; do

        echo "Num   Rootfs name                                    Date        Size"
        echo "-----------------------------------------------------------------------"

        # Show filesystems on server in columns
        for (( i = 0 ; i < ${#rootfs_links_array[@]} ; i++ ))
        do
            saveIFS=$IFS
            IFS=$'\n'
            NUM=$(printf "%-3s" "$i") 
            echo ${rootfs_links_array[$i]} | sed -e "s/\[ \]/$NUM/g" | column
            IFS=$saveIFS
        done

        # Read choice from user
        echo ""
        echo -n "Type the number [0-`expr ${#rootfs_links_array[@]} - 1`] of rootfs to download or 'f' to finish: "
        read choice

        # Check empty choice
        if [[ -z $choice ]] ; then
            echo "Your choice is not valid, please try again."
            echo ""
            sleep 2
            continue
        fi

        # Check for choice f = finish installing filesystems
        if [ $choice = f ] ; then
            echo "Finishing..."
            sleep 1
            break
        fi

      # Check that chosen number is on the list (0<=$choice<=max) and install
      if [ $choice -ge 0 ] ; then 
         if [ $choice -le `expr ${#rootfs_links_array[@]} - 1` ] ; then

            chosenbz2=$(echo ${rootfs_links_array[$choice]} | awk '{print $3}')
            chosen=$(echo $chosenbz2 | sed -e "s/.bz2//g")
            chosenbz2url=(${vnx_rootfs_repo}${chosenbz2})
            chosenmd5=$(echo ${rootfs_links_array[$choice]/.bz2/.md5} | awk '{print $3}')
            chosenmd5url=(${vnx_rootfs_repo}${chosenmd5}) 
            #cd $fsdir
            wget -N $chosenbz2url
            echo "Extracting $chosenbz2..."
            bunzip2 $chosenbz2
            # Create symbolic link if -l
            if [ "$create_sym_link" == "yes" ] ; then
               linkname=$(makeLinkName $chosen )
               echo "Creating simbolic link: $linkname"
               ln -s $chosen $linkname
               echo "$chosenbz2 successfully installed."
               echo ""
               sleep 1
            fi
            continue
         fi

         echo "Your choice is not valid, please try again."
         echo ""
         sleep 2
         continue
      fi
   
   done
}


# 
# Main
#
# Option -r vnx_rootfs_...qcow2.bz2 -> download specified fs and install
# Option -l -> Create symbolic link (create_sym_link="yes")
OPTIND=1
while getopts ":shl :r:" opt; do
    case $opt in

    l)
        #echo "-l was triggered" >&2
        create_sym_link="yes"
        ;;
    r)
        #echo "-r was triggered, Parameter: $OPTARG" >&2
        
        rootfs_bzname=$OPTARG
        
        NAME=$OPTARG
        if [ $( echo $NAME | egrep '\.bz2$' ) ]; then
            # Name with .bz2 extension
            rootfs_bzname=$NAME
            rootfs_name=$( echo $NAME | sed -e 's/\.bz2//' )
        else
            # Name without .bz2 extension
            rootfs_name=$NAME
            rootfs_bzname=${NAME}.bz2
        fi
        #echo "rootfs_name=$rootfs_name"
        #echo "rootfs_bzname=$rootfs_bzname"
        ;;
    s)
        #echo "-s was triggered, Parameter: $OPTARG" >&2
        echo "$HEADER"
        echo "Repository:    $vnx_rootfs_repo"
        echo ""
        show_rootfs_array
        exit 0
        ;;
    h)
        #echo "-h was triggered, Parameter: $OPTARG" >&2
        echo "$USAGE"
        exit 0
        ;;
    \?)
        echo "Invalid option: -$OPTARG" >&2
        echo "$USAGE"
        exit 1
        ;;
    :)
        echo "Option -$OPTARG requires an argument." >&2
        exit 1
        ;;

    esac
done

echo "$HEADER"
echo "vnx_download_rootfs: download, uncompress and optionally create links to "
echo "                     root filesystems for VNX to current directory"
echo ""
echo    "Repository:    $vnx_rootfs_repo"
echo -n "Current dir:   "
pwd 
echo ""

install

exit 0






